    /*
     * FASET Main code
     *
     * Blowfish implementation in CBC mode over BUFSIZ-16 length buffers
     * By detach hackaholic.org
     *
     * This program is free software; you can redistribute it and/or modify
     * it under the terms of the GNU General Public License as published by
     * the Free Software Foundation; either version 2 of the License, or
     * (at your option) any later version.
     */

#include <strings.h>
#include <unistd.h>
#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "blowfish.h"

enum mode {
  DECRYPT,
  ENCRYPT
};

  void
usage(char *prog)
{
  char syntax[]=
    "Usage: %s [-h] [-d | -e] [-k <passcode>] [infile] [outfile]\n"
    "\t-d         Decrypt mode (default)\n"
    "\t-e         Encrypt mode\n"
    "\t-k <>      Give key (default: getpass()) (max 56 byte -- 448 bits)\n"
    "\t-?         This help\n\n"
    "\t[infile]   Path to input file or - for STDIN (default)\n"
    "\t[outfile]  Path to output file (default: STDOUT)\n";

  printf(syntax, prog);
}

  int
main(int argc, char *argv[])
{
  int c;
  int mode = DECRYPT;
  unsigned char text[BUFSIZ];
  unsigned char decr[BUFSIZ];
  char *the_key=NULL;
  int infile, outfile;
  int n=1;

  assert((BUFSIZ % 8) == 0);

  opterr = 0;
  while ( (c=getopt( argc, argv, "dek:" )) != EOF)
      switch(c) {
          case 'd':
            mode = DECRYPT;
            break;
          case 'e':
            mode = ENCRYPT;
            break;
          case 'k':
            the_key = optarg;
            break;
          case '?':
            usage(argv[0]);
            exit(0);
      }

  if( argv[optind] ) {
    if( (infile = open (argv[optind], O_RDONLY)) == -1 ) {
      if (*(argv[optind]) == '-')
          infile = STDIN_FILENO;
      else {
        perror("[infile] open()");
        exit(1);
      }
    }
    if( argv[optind+1] && (outfile = open (argv[optind+1], (O_WRONLY|O_CREAT|O_EXCL), (S_IRUSR|S_IWUSR))) == -1 ) {
      perror("[outfile] open()");
      exit(1);
    }
    else if (!argv[optind+1])
        outfile = STDOUT_FILENO;
  } else {
      infile = STDIN_FILENO;
      outfile = STDOUT_FILENO;
  }

  if (!the_key && !(the_key = getpass( "Passcode: ")) ) {
   perror("getpass()");
   exit(1);
  }

  setkey(the_key, strlen(the_key));

  switch (mode) {
      case ENCRYPT:
        bzero (text, sizeof(text));
        // reserve two blocks, one for offset, one for padding data
        while( (n = read (infile, text+8, BUFSIZ-16)) > 0 ){
          n = encrypt(text, n);
          if (n)
              write(outfile, text, n);
          bzero (text, sizeof(text));
        }
        close(infile);
        close(outfile);

        exit(0);
      case DECRYPT:
        bzero (text, sizeof(text));
        bzero (decr, sizeof(text));
        while ( (n = read (infile, text, BUFSIZ)) > 0 ) {
          n = decrypt(text, decr, n);
          write(outfile, decr, n);
          bzero (text, n);
          bzero (decr, n);
        }
        close(infile);
        close(outfile);

  }

  exit(0);
}
